/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.plugin.editor;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.ISafeRunnable;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.SafeRunner;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.*;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.eclipse.ui.part.MultiPageEditorSite;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.views.contentoutline.*;
import org.eclipse.wst.json.core.contenttype.ContentTypeIdForJSON;
import org.eclipse.wst.sse.ui.StructuredTextEditor;
import org.eclipse.swt.custom.ScrolledComposite;

import com.fasterxml.jackson.core.JsonProcessingException;

import eu.DECIDEh2020.appManager.AppDescriptionFactory;
import eu.DECIDEh2020.appManager.AppDescriptionHelper;
import eu.DECIDEh2020.appManager.models.AppDescription;
import eu.DECIDEh2020.architect.plugin.Activator;
import eu.DECIDEh2020.architect.plugin.layoutComponents.NfrPageComposite;
import eu.DECIDEh2020.architect.plugin.layoutComponents.ProjectPageComposite;



/**
 * An example showing how to create a multi-page editor.
 * This example has 3 pages:
 * <ul>
 * <li>page 0 contains a nested text editor.
 * <li>page 1 allows you to change the font used in page 2
 * <li>page 2 shows the words in page 0 in sorted order
 * </ul>
 */
public class MultiPageEditor extends MultiPageEditorPart implements IResourceChangeListener {
	private static final String EXTENSION_ID ="eu.DECIDEh2020.architect.extension";
	
	/** The text editor used in the json page. */
	private StructuredTextEditor editor;

	private AppDescription project;
	private int pageIndex=1;
	

	/** The composite used for the project page */
	private ProjectPageComposite ppage;
	private NfrPageComposite nfrPage;
	private List<Page> pages = new ArrayList<Page>();
	private IContentOutlinePage contentOutlinePage;
	private File file= null;
	/**
	 * Creates a multi-page editor example.
	 */
	public MultiPageEditor() {
		super();
		ResourcesPlugin.getWorkspace().addResourceChangeListener(this);
	}
		
	//add all pages from other plugins
	void evaluateExtensions() {
		
		IConfigurationElement[] config = Platform.getExtensionRegistry().getConfigurationElementsFor(EXTENSION_ID);
		evalConfigs(config);
	}
	void evalConfigs(IConfigurationElement[] config) {
		System.out.println("configs: " + config.length);
		try {
            for (IConfigurationElement e : config) {
                System.out.println("Evaluating extension");
                final Object o =
                        e.createExecutableExtension("class");
                final String title = e.getAttribute("PageName");
                
                if (o instanceof IPageBuilder) {
                	ISafeRunnable runnable = new ISafeRunnable() {
                        @Override
                        public void handleException(Throwable e) {
                            System.out.println("Exception in client");
                        }

                        @Override
                        public void run() throws Exception {
                           Page page = ((IPageBuilder) o).getPage(getContainer(),SWT.NONE, project);
                           pages.add(page);
                           
                           int index = addPage(page);
                           page.setIndex(index);
                   		setPageText(index, title);
                        }
                    };
                    SafeRunner.run(runnable);
                }
            }
        } catch (CoreException ex) {
            System.out.println(ex.getMessage());
        }
	}
	
	/**
	 * Creates the page of the multi-page editor,
	 * which contains a json editor.
	 */
	void createJSONPage() {
		try {
			editor = new StructuredTextEditor();
			
			int index = addPage(editor, getEditorInput());
			setPageText(index, editor.getTitle());


		} catch (PartInitException e) {
			ErrorDialog.openError(
					getSite().getShell(),
					"Error creating nested text editor",
					null,
					e.getStatus());
		}
		
	}
	/**
	 * Creates page 1 of the multi-page editor,
	 * which contains the project overview.
	 */
	void createProjectPage() {

		fromJSON();
		ppage = new ProjectPageComposite(getContainer(), project);
		int index = addPage(ppage);
		setPageText(index, "Project");
	}
	/**
	 * Creates page 2 of the multi-page editor,
	 * which shows the NFR Editor.
	 */
	void createPatternPage() {

		nfrPage = new NfrPageComposite(getContainer(), project);
		
		int index = addPage(nfrPage);
		setPageText(index, "NFR Editor");
	}
	
	/**
	 * Creates the pages of the multi-page editor.
	 */
	protected void createPages() {
		//add editor pages
		createJSONPage();
		createProjectPage();
		createPatternPage();
		//pages from other plugins
		evaluateExtensions();
		//set project overview as startpage
		setActivePage(1);
		pageIndex=1;
		refreshOutlineView();
	}
	/**
	 * The <code>MultiPageEditorPart</code> implementation of this 
	 * <code>IWorkbenchPart</code> method disposes all nested editors.
	 * Subclasses may extend.
	 */
	public void dispose() {
		ResourcesPlugin.getWorkspace().removeResourceChangeListener(this);
		super.dispose();
	}
	/**
	 * Saves the multi-page editor's document.
	 */
	public void doSave(IProgressMonitor monitor) {
		toJSON();
		getEditor(0).doSave(monitor);
	}
	/**
	 * Saves the multi-page editor's document as another file.
	 * Also updates the text for page 0's tab, and updates this multi-page editor's input
	 * to correspond to the nested editor's.
	 */
	public void doSaveAs() {
		IEditorPart editor = getEditor(0);
		editor.doSaveAs();
		setPageText(0, editor.getTitle());
		setInput(editor.getEditorInput());
	}
	/* (non-Javadoc)
	 * Method declared on IEditorPart
	 */
	public void gotoMarker(IMarker marker) {
		setActivePage(0);
		IDE.gotoMarker(getEditor(0), marker);
	}
	
	/**
	 * The <code>MultiPageEditorExample</code> implementation of this method
	 * checks that the input is an instance of <code>IFileEditorInput</code>.
	 */
	public void init(IEditorSite site, IEditorInput editorInput)
			throws PartInitException {
		if (!(editorInput instanceof IFileEditorInput))
			throw new PartInitException("Invalid Input: Must be IFileEditorInput");
		super.init(site, editorInput);
		
		 IFileEditorInput finput = (IFileEditorInput) editorInput;
		 file=finput.getFile().getRawLocation().makeAbsolute().toFile();
		 if(!file.exists()) {
			 throw new PartInitException("File does not exist");
		 }
		// manager = AppManager.open("", "", file);
	}
	
	/** 
	 * Allows saving on every page of the editor. However only the content of the 
	 * json page is saved which is written when switched to that page
	 **/
	public boolean isSaveAsAllowed() {
		return true;
	}

	/**
	 * Calculates the contents of the then visible page when the it is activated.
	 */
	protected void pageChange(int newPageIndex) {
		//if we switch _from_ the JSON, load it into the appDescription class
		super.pageChange(newPageIndex);
		if (pageIndex==0) {
			fromJSON();
		}
		
		switch (newPageIndex) {
		case 0:
			//if we switch _to_ JSON, load from ApPDesscription Object toJSON
			toJSON();
			break;
		case 1:
			ppage.refreshContent(project);
			break;
		case 2:
			nfrPage.refreshContent(project);
			break;
		default:
			//we are switching to a page from another plugin
			Page page;
			try {
				page = (Page) getControl(newPageIndex);
				page.open(project);
			} catch (ClassCastException e) {
				Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, "Open project page.", e));
			}			
		}
		pageIndex= newPageIndex;
	}
	
	/**
	 * Closes all project files on project close.
	 */
	public void resourceChanged(final IResourceChangeEvent event){
		if(event.getType() == IResourceChangeEvent.PRE_CLOSE){
			Display.getDefault().asyncExec(new Runnable(){
				public void run(){
					IWorkbenchPage[] pages = getSite().getWorkbenchWindow().getPages();
					for (int i = 0; i<pages.length; i++){
						if(((FileEditorInput)editor.getEditorInput()).getFile().getProject().equals(event.getResource())){
							IEditorPart editorPart = pages[i].findEditor(editor.getEditorInput());
							pages[i].closeEditor(editorPart,true);
						}
					}
				}            
			});
		}
	}
	
	
	
	
	
	/**
	 * Converts JSON into a project Object
	 */
	void fromJSON() {
		String editorText = editor.getDocumentProvider().getDocument(editor.getEditorInput()).get();
			try {
				
				project = AppDescriptionFactory.fromJson(editorText);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		if (project == null) {

			System.out.println("project is null after reading JSON");
		}

	}



	/**
	 * Converts a project Object into JSON
	 */
	void toJSON() {
		if (project != null) {
			
			// convert app descriptor object to json string
			String contents = "";
			try {
				AppDescriptionHelper adh = AppDescriptionHelper.create(project);
				contents = adh.getJson();
			} catch (JsonProcessingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			// get editor content
			String editorText = editor.getDocumentProvider().getDocument(editor.getEditorInput()).get();

			// if new json string is different from editor content, replace old with new
			if (!contents.equals(editorText))
				editor.getDocumentProvider().getDocument(editor.getEditorInput()).set(contents);

		}
	}
	
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	@Override
	public Object getAdapter(Class key) {
	    if (key.equals(IContentOutlinePage.class)) {
	        //return showOutlineView() ? getContentOutlinePage() : null;
	    	return getContentOutlinePage();
	    } else {
	        return super.getAdapter(key);
	    }
	}
	
	
	/**
	 * This accesses a cached version of the content outliner.
	 */
	public IContentOutlinePage getContentOutlinePage() {
	    if (contentOutlinePage == null) {
	        contentOutlinePage = (IContentOutlinePage) editor.getAdapter(IContentOutlinePage.class);
	        
	    }
	    return contentOutlinePage;
	}
	
	/**
	 * @see org.eclipse.ui.part.MultiPageEditorPart#createSite(org.eclipse.ui.IEditorPart)
	 */
	protected IEditorSite createSite(IEditorPart page) {
		IEditorSite site = null;
		if (page == editor) {
			site = new MultiPageEditorSite(this, page) {
				public String getId() {
					// Sets this ID so nested editor is configured for JSON source
					return ContentTypeIdForJSON.ContentTypeID_JSON + ".source"; //$NON-NLS-1$;
				}
			};
		}
		else {
			site = super.createSite(page);
		}
		return site;
	}


	public void refreshOutlineView() {
		//get the activePage
		IWorkbenchPage wp =  PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		//Find desired view by its visual ID
		IViewPart myView=wp.findView("org.eclipse.ui.views.ContentOutline");

		//Hide the view :
		wp.hideView(myView);
		try {
			//show the view again     
			PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView("org.eclipse.ui.views.ContentOutline");
		} catch (PartInitException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
	}
}
